!#========================================================================
!#                                                                       #
!# Software Name : UVSOR ver. 3.01                                       #
!#                                                                       #
!#      Sub Routine : WriteDownData_onto_Files_Eps                       #
!#                                                                       #
!#                                Written by T. Hamada 2006/4/28         #
!#                                                                       #
!#      Contact address :  IIS,The University of Tokyo RSS21 project     #
!#                                                                       #
!#"Multiscale Simulation System for Functional Analysis of Nanomaterials"#
!#                                                                       #
!#========================================================================
!
! $Id: WriteDownData_onto_Files_Epsilon.f90 376 2014-06-17 07:48:31Z jkoga $
   subroutine WriteDownData_onto_Files_Eps
     use m_Epsilon_ek,               only : nlo, wd_eps, dealloc_m_Epsilon &
                                        & , kv3_in_the_ek_process
     use m_Files,                    only : m_Files_close_nfeps, m_Files_close_nfepscont, m_Files_close_nfnlo
     use m_Kpoints,                  only : kv3
     call wd_eps
     call dealloc_m_Epsilon
     call m_Files_close_nfeps
     call m_Files_close_nfepscont
     if(nlo/=0) call m_Files_close_nfnlo
   end subroutine WriteDownData_onto_Files_Eps
