      module eigen_dc
      implicit NONE

       real(8) ::  flops, dgemm_time, dgemm_dummy(1:2)
       real(8) ::  p_time0, p_time2, p_time3
       real(8) ::  p_times, p_timez, p_timer

      end module eigen_dc

